#!/bin/sh
#
# $Id: go.sh,v 1.5 2004-10-14 14:17:09 pl Exp $
#
# Some script to do some interactive raytracing

die () {
	echo " <br /> ERROR: $@<br \>"
	exit 1
}

# Working directory
WD="$1"

# Pov scene file to render
POVFILE=`basename "$2"`

# Output image file name
OUTIMGFN="$3"

[ "$WD" ] || die "WD not set"
[ "$WD/$POVFILE" ] || die "POVFILE not set"
[ "$OUTIMGFN" ] || die "OUTIMGFN not set"

[ -d "$WD" ] || die "WD=$WD is not a directory"
[ -r "$POVFILE" ] || die "POVFILE=$POVFILE is not available"

[ "$WIDTH" ] || WIDTH=640
[ "$HEIGHT" ] || HEIGHT=480
[ "$CHUNKH" ] || CHUNKH=15

[ "$WIDTH" -gt 0 ] || die "WIDTH must be > 0"
[ "$HEIGHT" -gt 0 ] || die "HEIGHT must be > 0"
[ "$CHUNKH" -gt 0 ] || die "CHUNKH must be > 0"

[ $(( $HEIGHT % $CHUNKH )) -eq 0 ] || "HEIGHT % CHUNKH must be equal to 0"


TASKDIR="tasks"
IMGDIR="img" 

mkdir $TASKDIR $IMGDIR

rm -f "$OUTIMGFN"

#############################################################################
/bin/echo "* Launching rendering process - Please be patient, this may take some time to initialize..."
/bin/echo -ne "!!! WARNING !!! Please do not try to reload this page manually as it will actually queue a new rendering process \n\n" 

/bin/echo -ne " POVRAYDEMO: submitting tasks... "

for Y in $(seq 1 $CHUNKH $HEIGHT ) ; do

	cat >$WD/$TASKDIR/plot_$Y.sh <<EOF
#!/bin/sh

#SGE : stdout+stderr to dev null
#$ -o /dev/null
#$ -e /dev/null

cd "$WD"

logger "Doing $POVFILE $Y / $HEIGHT"

/usr/bin/povray +L/usr/share/doc/povray/examples/advanced/blocks +L/home/pov/scenes/library +L/usr/share/fonts/truetype/msttcorefonts +I$WD/$POVFILE  +FT +W$WIDTH +H$HEIGHT +SR$(($Y)) +ER$(($Y+$(($CHUNKH - 1)))) +O$WD/$IMGDIR/img_${Y}.tga -D0 -Q9 +B1000 +R3 +A0.300

# NB: we put a semaphore file to know when the file is completely written
# NB2: there is most likely a window (with NFS) but it should be more narrow
hostname > $WD/$IMGDIR/img_${Y}.tga.done

logger "Done $POVFILE $Y / $HEIGHT"

EOF
	chmod +x $WD/$TASKDIR/plot_$Y.sh

	/usr/bin/oarsub -l/nodes=1/core=1 "$WD/$TASKDIR/plot_$Y.sh" 1>/dev/null 2>&1 
#&	
	if [ $? -ne 0 ]; then
		check_nodes=`/usr/bin/oarnodes|grep -i "No nodes to display"`
		if [ "$check_nodes" != "" ]; then
			die "could not submit job, please check that nodes are available"
		else
			die "could not submit job"
		fi
	fi
	/bin/echo -ne "#"

done

echo

NTASK=$(($HEIGHT / $CHUNKH))

#############################################################################
KEEPROLLING=1

cd "$IMGDIR"
sleep 15
while [ $KEEPROLLING -ne 0 ] ; do
	echo -n "Waiting for results... "
	FULLPIC=0

	# Wait for the first image chunks...
	if [ -f img_1.tga.done ] ; then

		# NB: 18 is the size of the TGA header

		# As all chunks should have the same size, get the size of the
		# first one to do some simple checks on the other (to know
		# whether they are have finished being written).
		FS="`stat -c %s img_1.tga`"
		DS=$(( $FS - 18 ))
		cat img_1.tga > tmp.tga
		> img.tga.doneall
		FULLPIC=1
		for I in $(seq $((1 + $CHUNKH)) $CHUNKH $HEIGHT) ; do
			if [ -f img_$I.tga.done ] && [ "`stat -c %s img_$I.tga`" -eq "$FS" ] ; then

				echo `cat img_$I.tga.done` >> img.tga.doneall

				# Copy the image contents in the merged image
				dd if=img_$I.tga bs=1 count=$DS skip=18 >> tmp.tga 2>/dev/null

				FULLPIC=$(($FULLPIC + 1))
				if [ $FULLPIC -eq $NTASK ] ; then
					KEEPROLLING=0
				fi
			else
				# If no picture is available for this chunk,
				# add some black instead
				dd if=/dev/zero bs=1 count=$DS skip=18 >> tmp.tga 2>/dev/null
			fi
		done
		
		# NB: we do these mv kludges to have a better atomicity wrt the
		# web server (a rename operation is most likely to be atomic
		# whereas a file copy is not)
		mv tmp.tga "$OUTIMGFN"
		TMPPNG="`dirname $OUTIMGFN`/`basename $OUTIMGFN .tga`-tmp.png"
		FINALPNG="`dirname $OUTIMGFN`/`basename $OUTIMGFN .tga`.png"
		convert "$OUTIMGFN" "$TMPPNG"
		mv "$TMPPNG" "$FINALPNG"

		# Try to illustrate the work progression by issuing a text progression bar
		/bin/echo -ne "["
		for FOO in $(seq 1 $NTASK) ; do 
			if [ $FOO -le $FULLPIC ] ; then
				/bin/echo -ne "#"
			else
				/bin/echo -ne " "
			fi
		done
		printf "] (%02d/%02d) " $FULLPIC $NTASK

		# Add some time length information	
		if [ ! $STARTTIME ] ; then
			STARTTIME=`date +%s`
		fi
		if [ $PREVDATE ] ; then
			NOW=`date +%s`
			printf " +%03ds = %03ds " $(($NOW - $PREVDATE)) $(($NOW - $STARTTIME))
		else
			printf " +%03ds = %03ds " 0 0
		fi
		PREVDATE=`date +%s`

		# Add some 'which machine did what' information	
		cat img.tga.doneall | ../machinestat.pl
	else
		echo "Nothing yet... Please wait for the tasks to start executing."
	fi
	sleep 1
done

#############################################################################
/bin/echo -e "Cleaning environment"
rm -rf "$WD"

